/*
 *  linux/include/asm-m68k/tekuart.h
 *
 *  Copyright (C) 2002,2003 Michael Mueller <malware@t-online.de>
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License. See the file COPYING in the main directory of this archive for
 * more details.
 */

#ifndef ASM_TEKUART
#define ASM_TEKUART

extern spinlock_t tek_uart_lock;

#define TEK_UART_BASE	(0xFF800000UL)

#define UART_IRQS 8

void tek_uart_init_IRQ(void);
int tek_uart_request_irq(unsigned int irq,
                          void (*handler)(int, void *, struct pt_regs *),
                          unsigned long flags,
                          const char *devname, void *dev_id);
void tek_uart_free_irq(unsigned int irq, void *dev_id);
void tek_uart_enable_irq(unsigned int irq);
void tek_uart_disable_irq(unsigned int irq);
int tek_uart_get_irq_list(char *buf);

void tek_uart_setup_timer(void (*handler)(int, void *, struct pt_regs *));
void tek_uart_putchar(int channel, int c, int waitloop);

void tek_uart_set_op(int nr);
void tek_uart_clr_op(int nr);

/*
 * Structures describing the register set of the Motorola 68681 DUART.
 */

struct tek_uart_rd {
	unsigned char	MRxA;
	unsigned short	_pad1;
	unsigned char	SRA;
	unsigned short	_pad2;
	unsigned char	BRG_Test;
	unsigned short	_pad3;
	unsigned char	RHRA;
	unsigned short	_pad4;
	unsigned char	IPCR;
	unsigned short	_pad5;
	unsigned char	ISR;
	unsigned short	_pad6;
	unsigned char	CTU;
	unsigned short	_pad7;
	unsigned char	CTL;
	unsigned short	_pad8;
	unsigned char	MRxB;
	unsigned short	_pad9;
	unsigned char	SRB;
	unsigned short	_pad10;
	unsigned char	x1x16_Test;
	unsigned short	_pad11;
	unsigned char	RHRB;
	unsigned short	_pad12;
	unsigned char	res1;
	unsigned short	_pad13;
	unsigned char	IP;
	unsigned short	_pad14;
	unsigned char	CounterStart;
	unsigned short	_pad15;
	unsigned char	CounterStop;
	unsigned short	_pad16;
};

struct tek_uart_wr {
	unsigned char	MRxA;
	unsigned short	_pad1;
	unsigned char	CSRA;
	unsigned short	_pad2;
	unsigned char	CRA;
	unsigned short	_pad3;
	unsigned char	THRA;
	unsigned short	_pad4;
	unsigned char	ACR;
	unsigned short	_pad5;
	unsigned char	IMR;
	unsigned short	_pad6;
	unsigned char	CRUR;
	unsigned short	_pad7;
	unsigned char	CTLR;
	unsigned short	_pad8;
	unsigned char	MRxB;
	unsigned short	_pad9;
	unsigned char	CSRB;
	unsigned short	_pad10;
	unsigned char	CRB;
	unsigned short	_pad11;
	unsigned char	THRB;
	unsigned short	_pad12;
	unsigned char	res1;
	unsigned short	_pad13;
	unsigned char	OPCR;
	unsigned short	_pad14;
	unsigned char	OutputSet;
	unsigned short	_pad15;
	unsigned char	OutputClear;
	unsigned short	_pad16;
};

union tek_uart {
	struct tek_uart_rd rd;
	struct tek_uart_wr wr;
};

typedef union tek_uart tek_uart_t;

#endif	/* defined(ASM_TEKUART) */

