#ifndef __LINUX_ASFS_FS_H
#define __LINUX_ASFS_FS_H

#include <linux/types.h>
#include <asm/byteorder.h>

#define asfs_debug(fmt,arg...) pr_debug(fmt,##arg)
//#define asfs_debug(fmt,arg...) printk(fmt,##arg)

/* some helper macros... */

#define from32be(t) ((t)=be32_to_cpu(t))
#define from16be(t) ((t)=be16_to_cpu(t))

#define ASFS_MAKE_ID(a,b,c,d) (((a)&0xff)<<24|((b)&0xff)<<16|((c)&0xff)<<8|((d)&0xff))

/* Amiga SFS defines */

#define ASFS_MAGIC 0xa0ff
#define ASFS_MAXFN 105

#define ASFS_ROOTID ASFS_MAKE_ID('S','F','S',0)

#define ASFS_DEFAULT_UID 0 
#define ASFS_DEFAULT_GID 0
#define ASFS_DEFAULT_MODE 0444 /* default permission bits for files, dirs have same permission, but with "x" set */ 

#define ASFS_STRUCTURE_VERISON (3)

#define OTYPE_HIDDEN      (1)
#define OTYPE_LINK        (64)
#define OTYPE_DIR         (128)

#define MSB_MASK (1ul << 31)

/* Each block has its own header with checksum and id, its called fsBlockHeader */

struct fsBlockHeader {
	u32 id;				/* 4 character id string of this block */
	u32 checksum;			/* The checksum */
	u32 ownblock;			/* The blocknumber of the block this block is stored at */
};

/* On-disk "super block", called fsRootBlock */

struct fsRootBlock {
	struct fsBlockHeader bheader;

	u16 version;			/* Version number of the filesystem block structure */
	u16 sequencenumber;		/* The Root with the highest sequencenumber is valid */

	u32 datecreated;		/* Creation date (when first formatted).  Cannot be changed. */
	u8 bits;			/* various settings, see defines below. */
	u8 pad1;
	u16 pad2;

	u32 reserved1[2];

	u64 firstbyte;			/* The first byte of our partition from the start of the disk. */

	u64 lastbyte;			/* The last byte of our partition, excluding this one. */

	u32 totalblocks;		/* size of this partition in blocks */
	u32 blocksize;			/* blocksize used */

	u32 reserved2[2];
	u32 reserved3[8];

	u32 bitmapbase;			/* location of the bitmap */
	u32 adminspacecontainer;	/* location of first adminspace container */
	u32 rootobjectcontainer;	/* location of the root objectcontainer */
	u32 extentbnoderoot;		/* location of the root of the extentbnode B-tree */
	u32 objectnoderoot;		/* location of the root of the objectnode tree */

	u32 reserved4[3];
};

/* On disk inode, called fsObject */

struct fsObject {
	u16 owneruid;
	u16 ownergid;
	u32 objectnode;
	u32 protection;

	union {
		struct {
			u32 data;
			u32 size;
		} file;

		struct {
			u32 hashtable;	/* for directories & root, 0 means no hashblock */
			u32 firstdirblock;
		} dir;
	} object;

	u32 datemodified;
	u8 bits;

	u8 name[0];
	u8 comment[0];
};

/* On disk block containging a number of fsObjects */

struct fsObjectContainer {
	struct fsBlockHeader bheader;

	u32 parent;
	u32 next;
	u32 previous;			/* 0 for the first block in the directory chain */

	struct fsObject object[0];
};

/* BTree structures, used to collect file data position on disk */

struct fsExtentBNode {
	u32 key;			/* data! */
	u32 next;
	u32 prev;
	u16 blocks;			/* The size in blocks of the region this Extent controls */
};

struct BNode {
	u32 key;
	u32 data;
};

struct BTreeContainer {
	u16 nodecount;
	u8 isleaf;
	u8 nodesize;			/* Must be a multiple of 2 */

	struct BNode bnode[0];
};

/* On disk block with BTreeContainer */

struct fsBNodeContainer {
	struct fsBlockHeader bheader;
	struct BTreeContainer btc;
};

#ifdef __KERNEL__

/* Not much now */

#endif /* __KERNEL__ */
#endif
